// MgApp.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGAPP_H
#define MGAPP_H

#include <SDL.h>

#include "MgThread.h"
#include "MgVideoSystem.h"
#include "MgEventFilter.h"
#include "MgMutex.h"
#include "MgTimerSystem.h"
#include "MgAudioSystem.h"
#include "MgNetSystem.h"

/*! \file MgApp.h
	Contains the MgApp class as well as libMgTk and SDL initializing routines.
*/
/*
	This new version of libMgTk will handle initializing SDL for you. In your main file, just
	derive from MgApp and reimplement some functions. You don't even have to write a main function
	or initialize SDL.
	
	Example:
	
	clas SomeApp : public MgApp
	{
	
	public:
	
		virtual void ArgvReceived(int argc, char ** argv);
		virtual void ReadyToRun();
		virtual bool QuitRequested();
	
	protected:
	
		virtual int Run();
		
	private:
	
		// some vars...
	};
	
	void SomeApp::ArgvReceived(int argc, char ** argv)
	{
		// do something
	}
	
	void SomeApp::ReadyToRun()
	{
		// do some initializing
	}
	
	bool SomeApp::QuitRequested()
	{
		// do some checking...
		// return true if you want to quit, or false not to quit
	}
	
	int SomeApp::Run()
	{
		// your main program loop
		// do a bunch of stuff in here...
	}
	
	// now do this, passing the name of your app class, and the flags for what we want
	// libMgTk to initialize.
	MGTK_MAIN(SomeApp, MG_VIDEO | MG_NET | MG_AUDIO)
*/


/*! Init the timer subsystem. */
#define MG_TIMER SDL_INIT_TIMER
/*! Init the audio subsystem. */
#define MG_AUDIO SDL_INIT_AUDIO
/*! Init the video subsystem. */
#define MG_VIDEO SDL_INIT_VIDEO
/*! Init the cdrom subsystem. */
#define MG_CDROM SDL_INIT_CDROM
/*! Init the joystick subsystem. */
#define MG_JOYSTICK SDL_INIT_JOYSTICK
/*! Init the net subsystem. */
#define MG_NET 0x00001000
/*! Do not let SDL catch fatal signals. */
#define MG_NO_PARACHUTE SDL_INIT_NOPARACHUTE
//! Pass this to have ArgvReceived() called even if 'argc' is 1.
#define MG_ARGV_RECEIVED_ANYWAY 0x10000000
/*! Pass this if you don't want an event loop outside of the main app thread.
	But remember, if you do this, any event filters you have running will not
	receive events unless you call MgApp::PumpEvents().
*/
#define MG_NO_EVENTTHREAD 0x20000000
#define MG_EVERYTHING SDL_INIT_EVERYTHING	// will grap MG_NET too

//! MgApp::AddFilter() flags.
enum
{
	MG_FILTER_SYSTEM = 1, 	//! The filter is a system wide filter (it will receive events even if you try to drop them)
	MG_FILTER_USER			//! The filter is a user level filter.
};

#include "MgBeginCode.h"

class MgAppThread;
class MgAppFilterHelper;

//! This contains the error codes returned by MgApp::Run(). You will probably never use them yourself.
enum
{
	MG_RUN_NO_ERROR = 0, /*!< No error. */
	MG_RUN_SDL = -1, /*!< SDL_Init() failed. */
	MG_RUN_NET = -2, /*!< Failed to start the networking code. (MgNet_Init() failed) */
	MG_RUN_THREAD = -3, /*!< Failed to create app thread. */
	MG_RUN_WRONG_SDL = -4, /*!< Wrong version of SDL installed. */
};
		

/*!
	MgApp class. Derive from this class.
*/
class MGDLL MgApp
{
public:
	//! Constructor. Does nothing.
	MgApp();
	//! Destructor. Does nothing.
	virtual ~MgApp();
	
	virtual int Run(Uint32 flags, const char * title, int argc, char ** argv);

	//! Get a pointer to the video system.
	static MgVideoSystem * Video();
	//! Get a pointer to the timer system.
	static MgTimerSystem * Timer();
	//! Get a pointer to the audio system.
	static MgAudioSystem * Audio();
	//! Get a pointer to the net system.
	static MgNetSystem * Net();
	
	//! Get current elapsed ticks.
	static Uint32 GetTicks();
	//! Delay for 'ms' milliseconds.
	static void Delay(Uint32 ms);
	
	//! Get a pointer to the app.
	static MgApp * App();
	
	//! Add event filter.
	virtual bool AddFilter(MgEventFilter * filter, Uint32 events, int type = MG_FILTER_USER);
	//! Remove a filter.
	virtual bool RemoveFilter(MgEventFilter * filter, int type = MG_FILTER_USER);
	//! Pause the filter system.
	virtual bool PauseFilter();
	//! Resume the filter system.
	virtual void ResumeFilter();

	//! This enum lists the event status for PumpEvents().
	enum EventStatus
	{
		MGA_OK = 0,			/*!< Keep on running. */
		MGA_LOCK_FAILED,	/*!< Failed to acquire the mutex. */
		MGA_QUIT			/*!< The quit event was received. Your app should quit now.*/
	};
	
	//! Pump the event filters.
	virtual EventStatus PumpEvents();
	
	//! Poll the event queue for an event. 
	virtual bool GetRawEvent(SDL_Event & event);
	//! Wait for an event to enter the queue and return it.
	virtual bool WaitRawEvent(SDL_Event & event);

protected:
	virtual void ReadyToRun();
	virtual void ArgvReceived(int argc, char ** argv);
	virtual bool QuitRequested();

	virtual int RunApp();

	
	//! Call this to see if we were quit.
	bool WasQuit();
	
	//! When your Run() method is done, call this at the end to quit the program and  not wait for the user to quit it.
	void Quit();

private:
	Uint32 fFlags;
	SDL_Event fRawEvent;
	bool fHaveRawEvent;

	bool fQuit;
	
	int EventLoop();
	
	MgAppThread * fLoopThread;
	MgAppFilterHelper ** fUserFilters;
	MgAppFilterHelper ** fSystemFilters;
	int fNumUserFilters;
	int fNumSystemFilters;
	MgMutex * fFilterLock;
	
	
	static MgVideoSystem * fVideo;
	static MgTimerSystem * fTimer;
	static MgAudioSystem * fAudio;
	static MgNetSystem * fNet;
	
	static MgApp * fApp;	// the app;
	
	friend class MgAppThread;
	friend class MgDefaultApp;
};

#include "MgEndCode.h"

/*!
	Put this in as your main function and pass it your app, the flags, and the title to
	be applied to the window if you are going to use the video functions.

	\param app An MgApp subclassed object.
	\param flags The MG_* flags.
	\param title The const char * to the window title. Can be NULL.
*/
#define MGTK_MAIN(app, flags, title) \
		int main(int argc, char ** argv) \
		{ \
			MgApp * libMgTkApplication = new app; \
			int ret = libMgTkApplication->Run(flags, title, argc, argv); \
			delete libMgTkApplication; \
			return ret; \
		}
#endif

/* this function returns true if the version of SDL on the users computer
 * is greater or equal to the minimum required version.
 */
MGDLL bool MgTkCheckVersion();
