// MgJoystick.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGJOYSTICK_H
#define MGJOYSTICK_H

/*! \file MgJoystick.h

	This file contains joystick handling code.
*/
#include "MgEventFilter.h"

#include "MgBeginCode.h"

/* ripped from SDL_joystick.h so i wouldn't have to include the header. */
struct _SDL_Joystick;
typedef struct _SDL_Joystick SDL_Joystick;

/*! This is the base class for MgJoystick and MgSJoystick. You can use the methods
	it contains to find out things about the joystick system.
*/
class MGDLL MgJoystickBase
{
public:	
	//! Find out how many joysticks there are attached to the system.
	static int Amount();
	//! Returns the name of a joystick, or NULL if no name was found.
	static const char * Name(int joystick);

	//! Enable joystick event polling.
	static void Enable();
	//! Disable joystick event polling. You should not do this if you are using the event based joystick.
	static void Disable();
	
	//! Get the number of axis controls on a joystick.
	int NumAxes();
	//! Get the number of trackballs on a joystick.
	int NumBalls();
	//! Get the number of POV hats on a joystick.
	int NumHats();
	//! Get the number of buttons on a joystick.
	int NumButtons();

	//! This enum lists the return codes for InitCheck().
	enum Error
	{
		NoError = 0,	//!< No error.
		OpenFailed,		//!< Failed to open joystick.
		AlreadyOpen,	//!< Joystick is already open.
		AddFailed,		//!< Failed to add the event filter to MgApp.
		NoMemory		//!< Ran out of memory.
	};
		
	//! Call this before using the joystick.
	MgJoystickBase::Error InitCheck() const;

private:
	MgJoystickBase(int joystick);
	virtual ~MgJoystickBase();

	SDL_Joystick * fJoy;
	
	/* checks to see if the received event was generated by our joystick */
	bool BelongsToUs(Uint8 which) const;
	
	MgJoystickBase::Error fError;
	
	friend class MgJoystick;
	friend class MgSJoystick;
};

/*! This is the event filter driven version of MgJoystick. Before you can
	use this though, you must open a joystick with MgJoystick:Open(). You
	are responsible for opening and closing the joysticks if you
	use the event driven version of MgJoystick. This class works exactly like
	MgMouse.
*/
class MGDLL MgJoystick : public MgEventFilter, public MgJoystickBase
{
public:
	//! Constructor.
	MgJoystick(int joystick);
	//! Destructor..
	virtual ~MgJoystick();

	virtual bool JoystickAxis(const MgJoyAxisEvent & e);
	virtual bool JoystickBall(const MgJoyBallEvent & e);
	virtual bool JoystickHat(const MgJoyHatEvent & e);
	virtual bool JoystickButton(const MgJoyButtonEvent & e);

	//! This enum lists the codes sent to MgJoystick::Activated().
	enum Type
	{
		Axis = 1,	//!< Joystick axis event.
		Ball,		//!< Joystick ball event.
		Hat,		//!< Joystick hat event.
		Button		//!< Joystick button event.
	};
	
protected:
	//! Override this method receive joystick update events.
	virtual void Activated(MgJoystick::Type code) = 0;

	//! Contains the joystick axis event data.
	MgJoyAxisEvent fAxisEvent;
	//! Contains the joystick ball event data.
	MgJoyBallEvent fBallEvent;
	//! Contains the joystick hat event data.
	MgJoyHatEvent fHatEvent;
	//! Contains the joystick button event data.
	MgJoyButtonEvent fButtonEvent;

private:
};

/*! The static joystick class */
class MGDLL MgSJoystick : public MgJoystickBase
{
public:
	//! Constructor.
	MgSJoystick(int joystick);
	//! Destructor.
	virtual ~MgSJoystick();
	
	//! This class holds the static joystick data. Do not delete any of the data this class returns!
	class MGDLL Status
	{
	public:
		/*! Returns the status of all the buttons on the joystick
			starting from 0 to MgSJoystick::NumButtons() - 1.
		*/
		const Uint8 * Buttons() const;
		/*! Returns the status of all the axes on the joystick
			starting from 0 to MgSJopystick::NumAxes() - 1.
		*/
		const Sint16 * Axes() const;
		/*! Returns the status of all the hats on the joystick
			starting from 0 to MgSJoystick::NumHats() - 1.
		*/
		const Uint8 * Hats() const;
		
		//! Simple class that contians the data for the balls on the joystick.
		class MGDLL Ball
		{
		public:
			/*! Change in the x direction. */
			int dx;
			/*! Change in the y direction. */
			int dy;
		};
		
		/*! Returns the status of all the balls on the joystick
			starting from 0 to MgSJoystick::NumBalls() - 1.
		*/
		const Ball * Balls() const;
	
	private:
		Status() { fButtons = NULL; fAxes = NULL; fHats = NULL; fBalls = NULL; }
		~Status() { delete [] fButtons; delete [] fAxes; delete [] fHats; delete [] fBalls; }

		Uint8 * fButtons, * fHats;
		Sint16 * fAxes;
		Ball * fBalls;
		
		friend class MgSJoystick;
	};
	
	//! Get the status of the joystick.
	const MgSJoystick::Status * GrabStatus();
	
	//! Return the last grabbed status.
	const MgSJoystick::Status * GetStatus() const;


private:
	Status * fStatus;
};

//===================================================================================
// Inline status implementation
//===================================================================================

inline const Uint8 *
MgSJoystick::Status::Buttons() const
{
	return fButtons;
}

inline const Sint16 *
MgSJoystick::Status::Axes() const
{
	return fAxes;
}

inline const Uint8 *
MgSJoystick::Status::Hats() const
{
	return fHats;
}

inline const MgSJoystick::Status::Ball *
MgSJoystick::Status::Balls() const
{
	return fBalls;
}


#include "MgEndCode.h"

#endif

