// MgMouse.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGMOUSE_H
#define MGMOUSE_H

/*! \file MgMouse.h
	Contains the mouse handling routines.
*/

#include "MgEventFilter.h"

#include "MgBeginCode.h"

/*! This is the event based mouse-handling class. When constructed, it
	adds itself to MgApp's event list (as a system level filter, so
	that all mouse events will get to it). When a mouse event is
	received, it invokes the protected method MgMouse::Activated() and
	passes a flag indication the event type.
*/
class MGDLL MgMouse : public MgEventFilter
{
public:
	//! Constructor.
	MgMouse();
	//! Destructor.
	virtual ~MgMouse();
	
	virtual bool MouseMotion(const MgMouseMotionEvent & e);
	virtual bool MouseButton(const MgMouseButtonEvent & e);

	/*! The type of mouse event. */
	enum Type
	{
		Up = 1,		//!< Mouse button released.
		Down,		//!< Mouse button pressed.
		Move		//!< Mouse moved.
	};
	
	//! Call this to see if the object was constructed properly.
	bool InitCheck() const;
	
protected:
	/*! Implement this method to catch mouse events. This method should
		execute as fast as possible since it will block the delivery
		of events to other filters if it runs too long. Be careful what
		you do here, since this method is garaunteed to run in a seperate
		thread.
	 	
	 	\param code This flag tells you what mouse event happened.
	 */
	virtual void Activated(MgMouse::Type code) = 0;
	
	/*! If you received MOUSE_UP or MOUSE_DOWN, this
		variable will contain the click event data.
	*/
	MgMouseButtonEvent fButtonEvent;
	/*! If you recieved MOUSE_MOVE, this variable will contain
		the move event data.
	*/
	MgMouseMotionEvent fMoveEvent;
	
private:
	bool fOK;	/* holds the InitCheck() value. */
};

/*! This is a static mouse-handling class. To update the mouse status,
	you need to call MgSMouse::GrabStatus().
*/
class MGDLL MgSMouse
{
public:
	//! Constructor.
	MgSMouse();
	//! Destructor.
	~MgSMouse();
	
	/*! A simple data class that contains the status of the mouse. */
	class MGDLL Status
	{
	public:
		//! X pos.
		int X() const;
		//! Y pos.
		int Y() const;
		//! Relative X movement.
		int XRel() const;
		//! Relative Y movement.
		int YRel() const;
	
		//! Pressed buttons.
		Uint8 Buttons() const;

	private:
		int fX, fY, fXRel, fYRel;
		Uint8 fButtons;
	
		friend class MgSMouse;
	};
	
	//! Grab the mouse status.
	MgSMouse::Status GrabStatus();
	
	//! Return the status from the last call to GrabStatus().
	MgSMouse::Status GetStatus();
	
	//! Mouse the mouse to the specified position. (Generates a mouse-move event.)
	void Warp(Uint16 x, Uint16 y);

private:
	MgSMouse::Status fStatus;
};

#include "MgEndCode.h"

/* mouse status implementation (inline) */

inline int 
MgSMouse::Status::X() const
{
	return fX;
}

inline int 
MgSMouse::Status::Y() const
{
	return fY;
}

inline int 
MgSMouse::Status::XRel() const
{
	return fXRel;
}

inline int 
MgSMouse::Status::YRel() const
{
	return fYRel;
}

inline Uint8 
MgSMouse::Status::Buttons() const
{
	return fButtons;
}


#endif

