// MgNet.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGNET_H
#define MGNET_H

#include "MgBaseNet.h"
//#include "MgNetLib.h"

#include "MgBeginCode.h"

/*! \file MgNet.h
	This file contains the MgNet class.
*/

class MGDLL MgNet
{
public:
	
	/*!
		Destructor.
	*/
	virtual ~MgNet(void);
	
	/*!
		Open a network socket on a given port.
   	
   		\param port The port to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	virtual bool Open(Uint16 port) = 0;
	
	/*!
		Open a network socket
   	
   		\param ip The MgNetIPAddress to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	virtual bool Open(MgNetIPAddress *ip) = 0;
	
	/*!
		Open a socket connection.
	
		\param sockSet The socket set to add the newly socket.
		\param ip.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	virtual bool Open(MgNetSocketSet *sockSet, MgNetIPAddress *ip) = 0;
	
	/*!
		Open a socket connection.
	
		\param sockSet The socket set to add the newly socket.
		\param port.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	virtual bool Open(MgNetSocketSet *sockSet,Uint16 port) = 0;
	
	/*!
		Get a socket set.
		
		\retval The socket set.
	*/
	virtual MgNetSocketSet *GetSocketSet(void) = 0;
	
	/*! 
		Get the IP address of the remote system associated with the socket.
   		If the socket is a server socket, this function returns NULL.
   		
   		\retval The current MgNetIPAddress.
	*/
	virtual MgNetIPAddress * GetPeerAddress(void) = 0;
	
	/*!
		Send data over the non-server socket
   		
   		\param data A pointer to any kind type of data.
   		\param len Number of bytes sent over the network.
   		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
	*/
	virtual bool Send(void *data, int len) = 0;
	
	/*!
		Receive data from the non-server socket
		
		\param data A pointer to any kind type of data.
		\param maxlen Try to get up to maxlen bytes of data.
		\retval false if it fails.
		\retval true otherwise.
	*/
	virtual bool Receive(void *data,int maxlen) = 0;
	
	/*!
		Handle the client connection. Just pass NULL to the sock param will call the callbabck function with the sockSet.
		
		\param func The callback function calls to handle the client connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	virtual int HandleClient(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data) = 0;
	
	/*!
		Handle the server connection. Just pass NULL to the sock param will call the callbabck function with the sockSet.
		
		\param func The callback function calls to handle the server connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	virtual int HandleServer(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data) = 0;
	
	/*!
		Check  for events on a socket set.
		
		\param timeout The time which the system will wait for.
		\retval false If an error occured during the listening.
		\retval true otherwise.
	*/
	virtual bool CheckSocketSet(Uint32 timeout) = 0;
	
	/*!
		Close socket
	*/
	virtual void Close(void) = 0;

protected:
	MgNetIPAddress ipAddress;
	MgNetSocketSet * sockSet;
	int (*clientCallback)(MgNetGenericSocket *sock,void *);
	int (*serverCallback)(MgNetGenericSocket *sock,void *);
	
public:

	/*!
		Write a 16 bit value to network packet buffer
		
		\param value The value to be written.
		\param area.
	*/
	static void Write16(Uint16 value,void *area);
	
	/*!
		Write a 32 bit value to network packet buffer
		
		\param value The value to be written.
		\param area.
	*/
	static void Write32(Uint32 value,void *area);

	/*!
		Read a 16 bit value from network packet buffer
		
		\param area.
	*/
	static void Read16(void *area);
	
	/*!
		Read a 32 bit value from network packet buffer
		
		\param area.
	*/
	static void Read32(void *area);
	
	/*!
		Resolve an host name and port to an IP address in network form.	
		
		\param ip to store the information.
		\param port.
		\param host. If it is NULL, ip.host will be INADDR_ANY.
		\retval 0 if it succeeds
		\retval -1 otherwise and ip.host will be INADDR_NONE.
	*/
	static int ResolveHost(MgNetIPAddress *ip,char *host, Uint16 port);
	
	/*!
		Resolve an ip address to a host name in canonical form.
		
		\param ip.
		\retval the name of the Host.
	*/
	static char *ResolveIP(MgNetIPAddress *ip);		
};


#include "MgEndCode.h"

#endif

