// MgNetTCP.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGTCPNET_H
#define MGTCPNET_H

#include "MgNet.h"
#include "MgBeginCode.h"

/*! \file MgTCPNet.h
	This file contains the MgTCPNet class.
*/

/*!
	Use this class for TCP networking.
*/
class MGDLL MgTCPNet : public MgNet
{
public:
	
	/*!
		Default constructor. 
	*/
	explicit MgTCPNet(void);
	
	/*!
		Constructor which allocating socket set.
		
		\param maxSocketSet.
	*/
	MgTCPNet(int maxSocketSet);
	
	/*!
		Destructor. Closes the socket.
	*/
	~MgTCPNet(void);
	
	/*!
		Open a TCP network socket on a given port.
   	
   		\param port The port to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(Uint16 port);
	
	/*!
		Open a network socket
   	
   		\param port The port to connect.
   		\param host The host to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(Uint16 port,char *host);
	
	/*!
		Open a TCP network socket
   		If ip.host is INADDR_NONE, this creates a local server socket on the 
   		given port, otherwise a TCP connection to the remote host and port is
   		attempted.  The address passed in should already be swapped to network
   		byte order (addresses returned from SDLNet_ResolveHost() are already
   		in the correct form).
   	
   		\param ip The MgNetIPAddress to connect.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetIPAddress *ip);
	
	/*!
		Open a TCP socket connection.
	
		\param sockSet The socket set to add the newly TCP socket.
		\param ip.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetSocketSet *sockSet,MgNetIPAddress *ip);
	
	/*!
		Open a TCP socket connection.
	
		\param sockSet The socket set to add the newly TCP socket.
		\param port.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetSocketSet *sockSet, char * host, Uint16 port);
	
	/*!
		Open a TCP socket connection.
	
		\param sockSet The socket set to add the newly TCP socket.
		\param port.
		\param host.
		\retval false Connection failed.
		\retval true Connection succedeed.	
	*/
	bool Open(MgNetSocketSet *sockSet,Uint16 port,const char *host);
	
	/*!
		Accept an incoming connection on the server socket.
		
		\param tcpIncomingSock The newly created socket will be stored in here.
		\retval false Accept failed, then tcpIncomingSock is NULL.
		\retval true Accept succedeed.
	*/
	bool Accept(MgNetTCPSocket *tcpIncomingSock);
	
	/*!
		Get a socket set.
		
		\retval The socket set.
	*/
	MgNetSocketSet *GetSocketSet(void);
	
	/*! 
		Get the IP address of the remote system associated with the socket.
   		If the socket is a server socket, this function returns NULL.
   		
   		\retval The current MgNetIPAddress.
	*/
	MgNetIPAddress * GetPeerAddress(void);
	
	/*!
		Send data over the non-server socket
   		
   		\param data A pointer to any kind type of data.
   		\param len Number of bytes sent over the network.
   		\retval false if an error occurred or connection was closed. 
   		\retval true otherwise.
	*/
	bool Send(void *data, int len);
	
	/*!
		Receive data from the non-server socket
		
		\param data A pointer to any kind type of data.
		\param maxlen Try to get up to maxlen bytes of data.
		\retval false if it fails.
		\retval true otherwise.
	*/
	bool Receive(void *data,int maxlen);
	
	/*!
		Handle the client connection.
		
		\param func The callback function calls to handle the client connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	int HandleClient(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data);
	
	/*!
		Handle the server connection.
		
		\param func The callback function calls to handle the server connection.
		\param sock A pointer to a TCP or UDP network socket.
		\param data The data pointer.
		\return The callback returned value.
	*/
	int HandleServer(int (*callback)(MgNetGenericSocket *,void *), MgNetGenericSocket * sock, void * data);
	
	/*!
		Check  for events on a socket set.
		
		\param timeout The time which the system will wait for.
		\retval false If an error occured during the listening.
		\retval true otherwise.
	*/
	bool CheckSocketSet(Uint32 timeout);
	
	/*!
		Close the socket.
	*/
	void Close(void);
	
private:
	MgNetTCPSocket * tcpSock;
};

#include "MgEndCode.h"

#endif